import {
	CanActivate,
	ExecutionContext,
	ForbiddenException,
	Injectable,
	UnauthorizedException,
} from '@nestjs/common';
import { GqlExecutionContext } from '@nestjs/graphql';
import { PrismaService } from '../prisma/prisma.service';
import { Request } from 'express';
import { AgentFlags } from '../resolvers/models/agent.model';

@Injectable()
export class AuthGuard implements CanActivate {
	constructor(private prismaService: PrismaService) {}

	async canActivate(context: ExecutionContext): Promise<boolean> {
		const ctx = GqlExecutionContext.create(context);
		const req: Request = ctx.getContext().req;

		const token = req.headers.authorization;

		if (!token) throw new UnauthorizedException();

		const accessToken = await this.prismaService.accessToken.findFirst({
			where: { authToken: token },
		});

		if (!accessToken) throw new UnauthorizedException();

		if (accessToken.role === 'group') {
			req.group = await this.prismaService.group.findFirst({
				where: { id: accessToken.entityId },
			});
			req.isAdmin = false;
		} else {
			req.agent = await this.prismaService.agent.findFirst({
				where: { id: accessToken.entityId },
			});
			req.isAdmin =
				(req.agent.flags & AgentFlags.ADMIN) === AgentFlags.ADMIN;
		}

		if (!req.group && !req.agent)
			throw new ForbiddenException('Entity not found');

		return true;
	}
}
