import { Field, GraphQLISODateTime, ObjectType } from '@nestjs/graphql';
import { GraphQLFloat, GraphQLInt } from 'graphql/type';
import { AgentModel } from './agent.model';

@ObjectType('EntryImage')
export class EntryImageModel {
	@Field()
	id: string;

	@Field(() => GraphQLInt)
	width: number;

	@Field(() => GraphQLInt)
	height: number;
}

@ObjectType('EntryLocation')
export class EntryLocationModel {
	@Field(() => GraphQLFloat)
	lat: number;

	@Field(() => GraphQLFloat)
	lon: number;
}

@ObjectType('Entry')
export class EntryModel {
	@Field()
	id: string;

	@Field()
	private: boolean;

	@Field()
	unlocked: boolean;

	@Field(() => GraphQLISODateTime)
	createdAt: Date;

	@Field({ nullable: true })
	content?: string;

	@Field(() => EntryImageModel, { nullable: true })
	image?: EntryImageModel;

	@Field(() => EntryLocationModel, { nullable: true })
	location?: EntryLocationModel;

	@Field(() => AgentModel)
	agent?: AgentModel;

	@Field({ nullable: true })
	deleted?: boolean;
}
