import { Injectable } from '@nestjs/common';
import { Entry } from '@prisma/client';
import { EntryModel } from '../resolvers/models/entry.model';
import { PrismaService } from '../prisma/prisma.service';

@Injectable()
export class EntryService {
	constructor(private readonly prisma: PrismaService) {}

	public async getAccessibleEntries(
		groupId: string,
		agentId?: string,
	): Promise<EntryModel[]> {
		agentId = agentId ?? undefined;

		const [public_entries, unlocked_entries, locked_entries] =
			await Promise.all([
				this.prisma.entry.findMany({
					where: {
						private: false,
						agentId,
					},
				}),
				this.prisma.entry.findMany({
					where: {
						private: true,
						unlockedBy: {
							some: { id: groupId },
						},
						agentId,
					},
				}),
				this.prisma.entry.findMany({
					where: {
						private: true,
						unlockedBy: {
							none: { id: groupId },
						},
						agentId,
					},
				}),
			]);

		return [
			...public_entries.map(this.unlockedEntry),
			...unlocked_entries.map(this.unlockedEntry),
			...locked_entries.map(this.lockedEntry),
		];
	}

	public lockedEntry(entry: Entry): EntryModel {
		return {
			id: entry.id,
			private: true,
			createdAt: entry.createdAt,
			image: null,
			location: null,
			content: null,
			unlocked: false,
		};
	}

	public unlockedEntry(entry: Entry): EntryModel {
		return {
			...entry,
			image: entry.image_id
				? {
						id: entry.image_id,
						width: entry.image_width,
						height: entry.image_height,
				  }
				: null,
			location: entry.lat
				? {
						lat: entry.lat,
						lon: entry.lon,
				  }
				: null,
			unlocked: true,
		};
	}

	public rawEntry(entry: Entry): EntryModel {
		return {
			...entry,
			image: entry.image_id
				? {
						id: entry.image_id,
						width: entry.image_width,
						height: entry.image_height,
				  }
				: null,
			location: entry.lat
				? {
						lat: entry.lat,
						lon: entry.lon,
				  }
				: null,
			unlocked: !entry.private,
		};
	}
}
